unit UnitPing;

interface

function inet_addr(DotAddress: PChar): Integer; stdcall;
function PingStatusToStr(StatusCode: Integer): PChar;
function Ping(
  const DotAddress: PChar;
  const PacketSize_Byte: Integer;
  const ReceiveTimeout_mS: Integer;
  var Reply_StatusStr: PChar;
  var Reply_BytesReceived: Integer;
  var Reply_RoundTripTime_mS: Integer;
  var Reply_TTL: Integer): Integer;

implementation

uses
  Windows, SysUtils, UnitICMP;

function inet_addr; external 'wsock32.dll';

function PingStatusToStr(StatusCode: Integer): PChar;
begin
  case StatusCode of
    IP_SUCCESS              : Result := 'IP_SUCCESS';
    IP_BUF_TOO_SMALL        : Result := 'IP_BUF_TOO_SMALL';
    IP_DEST_NET_UNREACHABLE : Result := 'IP_DEST_NET_UNREACHABLE';
    IP_DEST_HOST_UNREACHABLE: Result := 'IP_DEST_HOST_UNREACHABLE';
    IP_DEST_PROT_UNREACHABLE: Result := 'IP_DEST_PROT_UNREACHABLE';
    IP_DEST_PORT_UNREACHABLE: Result := 'IP_DEST_PORT_UNREACHABLE';
    IP_NO_RESOURCES         : Result := 'IP_NO_RESOURCES';
    IP_BAD_OPTION           : Result := 'IP_BAD_OPTION';
    IP_HW_ERROR             : Result := 'IP_HW_ERROR';
    IP_PACKET_TOO_BIG       : Result := 'IP_PACKET_TOO_BIG';
    IP_REQ_TIMED_OUT        : Result := 'IP_REQ_TIMED_OUT';
    IP_BAD_REQ              : Result := 'IP_BAD_REQ';
    IP_BAD_ROUTE            : Result := 'IP_BAD_ROUTE';
    IP_TTL_EXPIRED_TRANSIT  : Result := 'IP_TTL_EXPIRED_TRANSIT';
    IP_TTL_EXPIRED_REASSEM  : Result := 'IP_TTL_EXPIRED_REASSEM';
    IP_PARAM_PROBLEM        : Result := 'IP_PARAM_PROBLEM';
    IP_SOURCE_QUENCH        : Result := 'IP_SOURCE_QUENCH';
    IP_OPTION_TOO_BIG       : Result := 'IP_OPTION_TOO_BIG';
    IP_BAD_DESTINATION      : Result := 'IP_BAD_DESTINATION';
    IP_ADDR_DELETED         : Result := 'IP_ADDR_DELETED';
    IP_SPEC_MTU_CHANGE      : Result := 'IP_SPEC_MTU_CHANGE';
    IP_MTU_CHANGE           : Result := 'IP_MTU_CHANGE';
    IP_UNLOAD               : Result := 'IP_UNLOAD';
    IP_GENERAL_FAILURE      : Result := 'IP_GENERAL_FAILURE';
  else
    Result := nil;
  end;
end;

function Ping(
  const DotAddress: PChar;
  const PacketSize_Byte: Integer;
  const ReceiveTimeout_mS: Integer;
  var Reply_StatusStr: PChar;
  var Reply_BytesReceived: Integer;
  var Reply_RoundTripTime_mS: Integer;
  var Reply_TTL: Integer): Integer;
const
  INADDR_NONE = -1;
var
  Reply: TICMP_Echo_Reply;
  EchoString: PChar;
  I: Integer;
  S: string;
  IPAddress: TIPAddr;
  ICMPPort: THandle;
begin
  IPAddress := inet_addr(DotAddress);
  if (IPAddress = INADDR_NONE) then
  begin
    Result := -1;
    Exit;
  end;
  ICMPPort := IcmpCreateFile();
  if (ICMPPort = INVALID_HANDLE_VALUE) then
  begin
    Result := -1;
    Exit;
  end;

  S := '';
  for I := 0 to PacketSize_Byte - 1 do
    S := S + Chr(1);
  EchoString := PChar(S);

  IcmpSendEcho(
    ICMPPort,
    IPAddress,
    EchoString,
    Length(EchoString),
    nil,
    @Reply,
    SizeOf(Reply),
    ReceiveTimeout_mS
    );
    
  IcmpCloseHandle(ICMPPort);

  case Reply.Status of
    0:
    begin
      Reply_StatusStr := 'IP_SUCCESS';
      Reply_BytesReceived := Reply.DataSize;
      Reply_RoundTripTime_ms := Reply.RoundTripTime;
      Reply_TTL := Reply.Options.Ttl;
      Result := 0;
    end;
    11010:
    begin
      Reply_StatusStr := 'IP_REQ_TIMED_OUT';
      Reply_BytesReceived := -1;
      Reply_RoundTripTime_ms := -1;
      Reply_TTL := -1;
      Result := 1;
    end;
    else
    begin
      Reply_StatusStr := PingStatusToStr(Reply.Status);
      Reply_BytesReceived := Reply.DataSize;
      Reply_RoundTripTime_ms := Reply.RoundTripTime;
      Reply_TTL := Reply.Options.Ttl;
      Result := Reply.Status;
    end;
  end;
end;

end.
